;**************************************************
;  The following Library of procedures are:
;
;     Copyrighted (c) 1993 Micro-Phyla Systems
;       All Rights Reserved
;
;  Author: John B. Moore
;  CIS#: 71333,1775
;  This code is licensed to the holder to be used
;  for their personal application development.  It
;  cannot be used to produce a commercial add-in 
;  that duplicates the function and purpose of
;  these utilities.
;**************************************************
;**************************************************
;                     TracTuner System Procs
;  Date: 9/6/93              Version 2
;**************************************************


; =================================================
; 09-06-93
; Creates TracTuner windows
; -------------------------------------------------
PROC TracTunerCreateWindows_u()
PRIVATE Procname.a, attrib_bag, h
  Procname.a = "TracTunerCreateWindows_u"
  ; We need three windows: Triggers, Procs, Watch
  ECHO OFF
  h = SaveWindowHandle_n()
  ; Reset video
  MENU {} {Video} {D: EGA/VGA: 80x43/50}
  ; Set up basic attributes
  DYNARRAY attrib_bag[]
    attrib_bag["CANCLOSE"]     = False
    attrib_bag["CANMAXIMIZE"]  = False
    attrib_bag["CANRESIZE"]    = False
    attrib_bag["CANMOVE"]      = False
    attrib_bag["FLOATING"]     = True
    attrib_bag["HASSHADOW"]    = False
    attrib_bag["HASFRAME"]     = False
    attrib_bag["MAXIMIZED"]    = False
    attrib_bag["CANVASHEIGHT"] = 200

    ; Triggers window
    attrib_bag["TITLE"]     = "Triggers"
    attrib_bag["ORIGINCOL"] = 0
    attrib_bag["ORIGINROW"] = 24
    attrib_bag["WIDTH"]     = 40
    attrib_bag["HEIGHT"]    = 12
    attrib_bag["STYLE"]     = 31  ; white on blue
    WINDOW CREATE ATTRIBUTES attrib_bag
      TO g_handle_bag["TRIGGERS"]
    PAINTCANVAS ATTRIBUTE 31 ALL

    ; Procs window
    attrib_bag["TITLE"]     = "Procs"
    attrib_bag["ORIGINCOL"] = 40
    attrib_bag["ORIGINROW"] = 24
    attrib_bag["WIDTH"]     = 40
    attrib_bag["HEIGHT"]    = 12
    attrib_bag["STYLE"]     = 111  ; White on brown

  WINDOW CREATE ATTRIBUTES attrib_bag
    TO g_handle_bag["PROCS"]
  PAINTCANVAS ATTRIBUTE 111 ALL

  ; Define dynarrays..
  DYNARRAY tr_trigger_bag[]
  DYNARRAY tr_procs_bag[]

  WINDOW SELECT h
  WINDOW SELECT h
  ECHO NORMAL

ENDPROC
;("TracTunerCreateWindows_u")

; =================================================
; 09-06-93
;
; -------------------------------------------------
PROC TracTunerDestroyWindows_u()
PRIVATE Procname.a, h
  Procname.a = "TracTunerDestroyWindows_u"
  IF ISASSIGNED(g_handle_bag["TRIGGERS"]) AND
     ISWINDOW(g_handle_bag["TRIGGERS"]) THEN
    ECHO OFF
    h = SaveWindowHandle_n()
    ; Close windows
    WINDOW SELECT g_handle_bag["TRIGGERS"]
    WINDOW SELECT g_handle_bag["TRIGGERS"]
    WINDOW CLOSE
    WINDOW SELECT g_handle_bag["PROCS"]
    WINDOW SELECT g_handle_bag["PROCS"]
    WINDOW CLOSE
    ; Release vars related to  TracTuner
    RELEASE VARS g_handle_bag["TRIGGERS"],
                 g_handle_bag["PROCS"],
                 tr_trigger_bag,
                 tr_procs_bag,
                 g_cycle_n

    ; Reset video
    MENU {} {Video} {C: EGA/VGA: 80x25}
    WINDOW SELECT h
    WINDOW SELECT h
    ; After returning to 80x25 a former max window
    ; will only take up half the screen
    WINMAX
  ENDIF
ENDPROC
;("TracTunerDestroyWindows_u")


; =================================================
; 09-06-93
; Main TracTuner proc
; -------------------------------------------------
PROC TracTuner_u(cycle_n, proc_a,
                 trigger_a, type_a, rval_n)
PRIVATE Procname.a, current_h
  Procname.a = "TracTuner_u"
  IF ISASSIGNED(TracTuner_l) AND TracTuner_l THEN
    IF NOT ISASSIGNED(g_cycle_n) THEN
      g_cycle_n = 0
    ENDIF
    ECHO OFF
    current_h = SaveWindowHandle_n()
    IF cycle_n <> g_cycle_n OR
       NOT ISASSIGNED(tr_trigger_bag) THEN
      ; Define or reset arrays
      DYNARRAY  tr_trigger_bag[]
      DYNARRAY  tr_procs_bag[]
    ENDIF

    TracTunerRefreshWindows_u(cycle_n, proc_a,
                         trigger_a, type_a, rval_n)

    SETCANVAS DEFAULT
    STYLE
    WINDOW SELECT current_h
    WINDOW SELECT current_h
    ; Update cycle
    g_cycle_n = cycle_n
  ENDIF
ENDPROC
;("TracTuner_u")

; =================================================
; 09-06-93
; Sets up or removes TracTuner
; -------------------------------------------------
PROC TracTunerSetup_n()
PRIVATE Procname.a
  Procname.a = "TracTunerSetup_n"
  ECHO OFF
  IF ISASSIGNED(TracTuner_l) THEN
    ; If you have mouse button canvas windows or 
    ; similar move those off screen at this point
    ; Example:
    ;   WaitMouseButtonPutaway_u(currentpage_n)
    TracTunerDestroyWindows_u()
    IF ISFILE("watch.sc") THEN
      RUN NOREFRESH "DEL watch.sc >>nul"
    ENDIF
    ; Move any floating windows back on screen
    ; to locate in proper position.
    ; Examples:
    ;  WINDOW MOVE g_handle_bag["SPEEDBAR"] TO 23,0
    ;  WaitMouseButtonsPlace_u(currentpage_n)
    RELEASE VARS TracTuner_l
  ELSE
    TracTuner_l = true
    ; If you have mouse button canvas windows or 
    ; similar, move those off screen at this point
    ; Example:
    ;   WaitMouseButtonPutaway_u(currentpage_n)
    IF ISFILE("watch.sc") THEN
      RUN NOREFRESH "DEL watch.sc >>nul"
    ENDIF
    TracTunerCreateWindows_u()
    ; Now move any floating windows back on screen
    ; to locate in proper position.
    ; Examples:
    ; WINDOW MOVE g_handle_bag["SPEEDBAR"] TO 23,0
    ; WaitMouseButtonsPlace_u(currentpage_n)
   ENDIF
   ECHO NORMAL
   RETURN 1
ENDPROC

; =================================================
; 09-07-93
; Writes to windows if info is present
; -------------------------------------------------
PROC TracTunerRefreshWindows_u(cycle_n, proc_a,
                         trigger_a, type_a, rval_n)
; type_a: "D" = default "C" = coherced
; proc_a: ""= no proc executed
; rval_n: current retval
PRIVATE Procname.a,
        el,
        n,
        count_n,
        basetrigger_a,
        abrtrigger_a,
        h
  Procname.a = "TracTunerRefreshWindows_u"
  h = SaveWindowHandle_n()
  ; Triggers
  basetrigger_a =  trigger_a
  abrtrigger_a  = TracTunerAbrTrigger_a(trigger_a)
  SETCANVAS g_handle_bag["TRIGGERS"]
  STYLE ATTRIBUTE 31
  IF g_cycle_n <> cycle_n THEN
    CLEAR
    PAINTCANVAS ATTRIBUTE 31 ALL
    WINDOW SELECT g_handle_bag["TRIGGERS"]
    WINDOW SCROLL g_handle_bag["TRIGGERS"] TO 0,0
    SETMARGIN OFF
    STYLE ATTRIBUTE 14
    @0,0 ?? FORMAT("W40,AC","* * TRIGGERS * *")
    @2,0
    STYLE ATTRIBUTE 31
  ENDIF
  SETMARGIN 3
  count_n = DYNARRAYSIZE(tr_trigger_bag)
  IF type_a = "C" THEN
    trigger_a = LOWER(trigger_a)
  ENDIF
  IF NOT ISBLANK(proc_a) THEN
    trigger_a =  CHR(251)+trigger_a
  ELSE
    trigger_a = " "+trigger_a
  ENDIF
  ; Place on screen
  ? STRVAL(count_n+1)+"-"+trigger_a
  ; Update cycle history
  tr_trigger_bag[count_n+1] = trigger_a

  ; Procs
  SETCANVAS g_handle_bag["PROCS"]
  STYLE ATTRIBUTE 111
  IF g_cycle_n <> cyclenumber_n THEN
    CLEAR
    PAINTCANVAS ATTRIBUTE 111 ALL
    WINDOW SELECT g_handle_bag["PROCS"]
    WINDOW SCROLL g_handle_bag["PROCS"] TO 0,0
    SETMARGIN OFF
    STYLE ATTRIBUTE 14
    @0,0 ??
    FORMAT("W40,AC","* * PROCEDURES TRIGGERED * *")
    @2,0
    STYLE ATTRIBUTE 111
  ENDIF
  SETMARGIN 3
  count_n = DYNARRAYSIZE(tr_procs_bag)
  proc_a = abrtrigger_a+proc_a+" = "+STRVAL(rval_n)
  ; Place on screen
  ? proc_a
  ; Update cycle history
  tr_procs_bag[count_n+1] = proc_a

  WINDOW SELECT h
  WINDOW SELECT h
ENDPROC
;("TracTunerRefreshWindows_u")


; =================================================
; 09-07-93
; Returns abbreviation of trigger
; -------------------------------------------------
PROC TracTunerAbrTrigger_a(eventype_a)
PRIVATE Procname.a
  Procname.a = "TracTunerAbrTrigger_a"
  SWITCH
    CASE eventype_a = "VALCHECK"      :RETURN "VC-"
    CASE eventype_a = "DEPARTFIELD"   :RETURN "DF-"
    CASE eventype_a = "DEPARTROW"     :RETURN "DR-"
    CASE eventype_a = "POSTRECORD"    :RETURN "PR-"
    CASE eventype_a = "DEPARTABLE"    :RETURN "DT-"
    CASE eventype_a = "DEPARTPAGE"    :RETURN "DP-"
    CASE eventype_a = "ARRIVEPAGE"    :RETURN "AP-"
    CASE eventype_a = "ARRIVEWINDOW"  :RETURN "AW-"
    CASE eventype_a = "ARRIVETABLE"   :RETURN "AT-"
    CASE eventype_a = "ARRIVEROW"     :RETURN "AR-"
    CASE eventype_a = "ARRIVEFIELD"   :RETURN "AF-"

    ; 4.5 triggers
    CASE eventype_a = "INSRECORD"     :RETURN "IR-"
    CASE eventype_a = "DELRECORD"     :RETURN "DL-"
    CASE eventype_a = "TOUCHFIELD"    :RETURN "TF-"
    CASE eventype_a = "ENDFIELDVIEW"  :RETURN "EF-"
    CASE eventype_a = "FIELDVIEW"     :RETURN "FV-"
    CASE eventype_a = "LOOKUP"        :RETURN "LK-"
    CASE eventype_a = "MENU"          :RETURN "MU-"
    CASE eventype_a = "UNDOCHANGE"    :RETURN "UC-"
    CASE eventype_a = "UNDODEL"       :RETURN "UD-"
    CASE eventype_a = "UNDOINS"       :RETURN "UI-"
    CASE eventype_a = "UNDOUNPOST"    :RETURN "UP-"
    CASE eventype_a = "CLOSE"         :RETURN "CL-"
    CASE eventype_a = "DONE"          :RETURN "DN-"
    CASE eventype_a = "OPEN"          :RETURN "OP-"
    CASE eventype_a = "FLYAWAY"       :RETURN "FA-"
    CASE eventype_a = "RECORDDELETED" :RETURN "RD-"
    CASE eventype_a = "RECORDCHANGED" :RETURN "RC-"
    CASE eventype_a = "ENDCYCLE"      :RETURN "EC-"
    CASE eventype_a = "REFRESH"       :RETURN "RF-"
    OTHERWISE : RETURN "NA-"
  ENDSWITCH
ENDPROC
;("TracTunerAbrTrigger_a")

;==================================================
; Last compiled:  9/07/93
;
;--------------------------------------------------
PROC TRACMenudef_n()
PRIVATE Procname_a
  Procname_a = "TRACMenudef_n"
  SHOWPOPUP " TracTuner System " CENTERED
    "~S~tart TracTuner" :
      "Start TracTuner System" : "start",
    "~C~lose TracTuner" :
      "Close TracTuner System" : "close",
    "~Q~uit TracTuner Menu" :
      "Close TracTuner menu" : "quit"
  ENDMENU TO choice_a
  SWITCH
    CASE choice_a = "quit" :
      RETURN 1
    CASE choice_a = "close" :
      TracTunerSetup_n()
    CASE choice_a = "start" :
      TracTunerSetup_n()
  ENDSWITCH
  RETURN 1
ENDPROC
;("TRACMenudef_n")
